__all__ = ['firstscreen.py', 'window.py', 'textview.py', 'notebook.py', 'linenumbers.py', 'handler.py', 'child.py',]

from tahrir.screen import Greet
from tahrir.window import *
from tahrir.textview import *
from tahrir.notebook import *
from tahrir.linenumbers import *
from tahrir.handler import *
from tahrir.child import *




from gi.repository import *
from tahrir.textview import *
from tahrir.linenumbers import *

class Child(Gtk.ScrolledWindow):
	def __init__(self, handler, content):
		super(Child, self).__init__()
		self.handler = handler
		self.content = content
		self.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
		self.set_layout()
		
	def set_layout(self):
		layout = Gtk.Box()
		layout.set_orientation(Gtk.Orientation.HORIZONTAL)
		self.doc = Doc(self, self.handler, self.content)
		self.doc.grab_focus()
		self.linenumbers = LineNumbers(self, self.doc.buffer)
		layout.pack_start(self.linenumbers, False, False, 0)
		layout.pack_start(self.doc, True, True, 0)
		layout.show_all()
		self.add_with_viewport(layout)
		
		
	




from gi.repository import Gtk
from tahrir.handler import *

class Window(Gtk.Window):
	def __init__(self, filename):
		super(Window, self).__init__()
		self.filename = filename
		self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
		self.set_default_size(700,550)
		self.set_title('txtr')
		self.handler = Handler(self)
		self.set_layout()
		self.show_all()
		self.connect("destroy", self.quit_app)
		
	def set_layout(self):
		layout = Gtk.Box() 
		layout.set_orientation(Gtk.Orientation.VERTICAL)
		toolbar = self.handler.toolbar
		notebook = self.handler.notebook
		layout.pack_start(toolbar, False, False, 0)
		layout.pack_start(notebook, True, True, 0)
		self.add(layout)
		#notebook.get_current_doc().grab_focus()
		notebook.new_tab(filename)
				
	def quit_app(self, widget):
		Gtk.main_quit()



from gi.repository import *

class LineNumbers(Gtk.TextView):
	def __init__(self, parent, docbuffer):
		super(LineNumbers, self).__init__()
		self.buffer = self.get_buffer()
		self.buffer.set_text('1')
		self.set_right_margin(5)
		self.set_editable(False)
		lines = docbuffer.get_line_count()
		self.set_lines(lines)
		self.set_can_focus(False)
		
	def set_lines(self, lines):
		self.buffer.set_text('')
		for l in range(1, lines+1):
			self.buffer.insert(self.buffer.get_end_iter(), '%d\n' % l)
		



from gi.repository import *
from tahrir.child import *

class Notebook(Gtk.Notebook):
	def __init__(self, handler):
		super(Notebook, self).__init__()
		self.handler = handler
		self.filenames = []
		#self.new_tab(None)
		self.connect('switch-page', self.page_changed)
		self.connect('page-removed', self.on_page_removed)
		self.connect('page-added', self.on_page_added)
		
	def new_tab(self, filename):
		label = Gtk.Box()
		label.set_orientation(Gtk.Orientation.HORIZONTAL)
		label.set_can_focus(False)
		if filename != None:
			titlestring = self.handler.split_filename(filename)
			content = self.handler.read_file(filename)
		else:
			titlestring = 'Untitled Document'
			content = ''
		self.filenames.append(filename)
		fileimage = Gtk.Image()
		fileimage.set_from_stock(Gtk.STOCK_FILE, Gtk.IconSize.MENU)
		title = Gtk.Label(titlestring)
		title.set_can_focus(False)
		image = Gtk.Image()
		image.set_from_stock(Gtk.STOCK_CLOSE, Gtk.IconSize.MENU)
		close = Gtk.Button()
		close.set_relief(Gtk.ReliefStyle.NONE)
		close.set_image(image)
		close.set_can_focus(False)
		label.pack_start(fileimage, False, False, 0)
		label.pack_start(title, True, True, 0)
		label.pack_start(close, False, False, 0)
		label.show_all()
		self.child = Child(self.handler, content)
		self.append_page(self.child, label)
		self.set_tab_reorderable(self.child, True)
		close.connect('clicked', self.on_click_close, self.child)
		self.show_all()
		self.set_current_page(-1)
		
	def update_window_title(self, filename):
		self.handler.set_window_title(filename)
		
	def update_tab_title(self, name):
		child = self.get_current_child()
		label = Gtk.Box()
		label.set_orientation(Gtk.Orientation.HORIZONTAL)
		title = Gtk.Label(name)
		image = Gtk.Image()
		image.set_from_stock(Gtk.STOCK_CLOSE, Gtk.IconSize.MENU)
		close = Gtk.Button()
		close.set_image(image)
		label.pack_start(title, True, True, 0)
		label.pack_start(close, False, False, 0)
		label.show_all()
		self.set_tab_label(child, label)
		
	def get_doc(self, pagenum):
		child = self.get_nth_page(pagenum)
		doc = child.doc
		return doc
		
	def get_current_child(self):
		page = self.get_current_page()
		child = self.get_nth_page(page)
		return child
		
	def get_current_doc(self):
		page = self.get_current_page()
		child = self.get_nth_page(page)
		doc = child.doc
		return doc
		
	def get_current_filename(self):
		page = self.get_current_page()
		filename = self.filenames[page]
		return filename
		
	def on_click_close(self, data, child):
		result = child.doc.has_unsaved_changes()
		if result == True:
			d = self.handler.confirm_exit()
			response = d.run()
			if response == -1:
				self.close_tab(child)
			elif response == 0:
				pass
			elif response == 1:
				self.handler.save_request()
			d.destroy()
		else:
			self.close_tab(child)
			
	def close_tab(self, child):
		page = self.page_num(child)
		filename = self.filenames[page]
		self.filenames.remove(filename)
		self.remove_page(page)
	
	def page_changed(self, notebook, page, pagenum):
		filename = self.filenames[pagenum]
		self.update_window_title(filename)
		currentdoc = self.get_current_doc()
		currentdoc.do_highlight()
		currentdoc.grab_focus()
				
	def on_page_removed(self, notebook, child, pagenum):
		if self.get_n_pages() == 0:
			self.handler.set_toolbar_actions(False)
		else:
			pass

	def on_page_added(self, notebook, child, page_num):
		if self.get_n_pages() == 1:
			self.handler.set_toolbar_actions(True)
			self.handler.set_save_icon(False)



from gi.repository import Gtk

class Toolbar(Gtk.Toolbar):
	def __init__(self, handler):
		super(Toolbar, self).__init__()
		self.handler = handler
		context = self.get_style_context()
		context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
		self.set_layout()
		self.set_can_focus(False)
		self.show_all()
		
	def set_layout(self):
		self.actions = {}
		#
		self.actions['new'] = Gtk.ToolButton()
		self.actions['new'].set_stock_id(Gtk.STOCK_NEW)
		self.actions['new'].set_tooltip_text('Create New Empty Document')
		self.actions['new'].connect('clicked', self.handler.on_click_new)
		self.insert(self.actions['new'], -1)
		#
		self.actions['open'] = Gtk.ToolButton()
		self.actions['open'].set_stock_id(Gtk.STOCK_OPEN)
		self.actions['open'].set_label('Open Doc')
		
		self.actions['open'].set_tooltip_text('Open Document')
		self.actions['open'].connect('clicked', self.handler.on_click_open)
		self.insert(self.actions['open'], -1)
		#
		self.actions['save'] = Gtk.ToolButton()
		self.actions['save'].set_stock_id(Gtk.STOCK_SAVE)
		self.actions['save'].set_label('Save Doc')
		self.actions['save'].set_tooltip_text('Save Document')
		self.actions['save'].set_sensitive(False)
		self.actions['save'].connect('clicked', self.handler.on_click_save)
		self.insert(self.actions['save'], -1)	
		#
		self.actions['separator'] = Gtk.SeparatorToolItem()
		self.insert(self.actions['separator'], -1)
		#
		self.actions['undo'] = Gtk.ToolButton()
		self.actions['undo'].set_stock_id(Gtk.STOCK_UNDO)
		self.actions['undo'].set_tooltip_text('Undo')
		self.actions['undo'].set_sensitive(False)
		self.actions['undo'].connect('clicked', self.handler.on_click_undo)
		self.insert(self.actions['undo'], -1)
		#
		self.actions['redo'] = Gtk.ToolButton()
		self.actions['redo'].set_stock_id(Gtk.STOCK_REDO)
		self.actions['redo'].set_tooltip_text('Redo')
		self.actions['redo'].set_sensitive(False)
		self.actions['redo'].connect('clicked', self.handler.on_click_redo)
		self.insert(self.actions['redo'], -1)
		#
		self.actions['separator1'] = Gtk.SeparatorToolItem()
		self.insert(self.actions['separator1'], -1)
		#
		self.actions['cut'] = Gtk.ToolButton()
		self.actions['cut'].set_stock_id(Gtk.STOCK_CUT)
		self.actions['cut'].set_tooltip_text('Cut text')
		self.actions['cut'].connect('clicked', self.handler.on_click_cut)
		self.insert(self.actions['cut'], -1)
		#
		self.actions['copy'] = Gtk.ToolButton()
		self.actions['copy'].set_stock_id(Gtk.STOCK_COPY)
		self.actions['copy'].set_tooltip_text('Copy text')
		self.actions['copy'].connect('clicked', self.handler.on_click_copy)
		self.insert(self.actions['copy'], -1)
		#
		self.actions['paste'] = Gtk.ToolButton()
		self.actions['paste'].set_stock_id(Gtk.STOCK_PASTE)
		self.actions['paste'].set_tooltip_text('Paste text')
		self.actions['paste'].connect('clicked', self.handler.on_click_paste)
		self.insert(self.actions['paste'], -1)
		#
		self.actions['spacer'] = Gtk.ToolItem()
		self.actions['spacer'].set_expand(True)
		self.insert(self.actions['spacer'], -1)
		#
		self.actions['search'] = Gtk.ToolItem()
		self.search_entry = Gtk.Entry()
		self.search_entry.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_FIND)
		self.search_entry.set_placeholder_text('Type to search ..')
		self.search_entry.connect('changed', self.handler.on_search)
		self.actions['search'].add(self.search_entry)
		self.insert(self.actions['search'], -1)
		#
		self.actions['search_forward'] = Gtk.ToolButton()
		self.actions['search_forward'].set_stock_id(Gtk.STOCK_GO_DOWN)
		direction = 'forward'
		self.actions['search_forward'].connect('clicked', self.handler.on_click_search_forward)
		self.insert(self.actions['search_forward'], -1)
		#
		self.actions['search_backward'] = Gtk.ToolButton()
		self.actions['search_backward'].set_stock_id(Gtk.STOCK_GO_UP)
		self.actions['search_backward'].connect('clicked', self.handler.on_click_search_backward)
		self.insert(self.actions['search_backward'], -1)
		#
		self.actions['about'] = Gtk.ToolButton()
		self.actions['about'].set_stock_id(Gtk.STOCK_ABOUT)
		self.actions['about'].connect('clicked', self.handler.on_click_about)
		self.insert(self.actions['about'], -1)
		#
		self.actions['settings'] = Gtk.ToolButton()
		self.actions['settings'].set_stock_id(Gtk.STOCK_PREFERENCES)
		self.actions['settings'].connect('clicked', self.handler.on_click_settings)
		self.insert(self.actions['settings'], -1)



from gi.repository import Gtk
from tahrir.handler import *
from tahrir.window import Window

class Greet(Gtk.Window):
	def __init__(self):
		super(Greet, self).__init__()
		self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
		self.set_default_size(500,400)
		self.set_layout()
		self.connect('destroy', self.destroy_window)
		
	def set_layout(self):
		box = Gtk.Box()
		box.set_orientation(Gtk.Orientation.VERTICAL)
		box.set_homogeneous(False)
		label = Gtk.Label("Please choose a source to view")
		button = Gtk.Button()
		image = Gtk.Image()
		image.set_from_stock(Gtk.STOCK_OPEN, Gtk.IconSize.DIALOG)
		button.set_image(image)
		button.set_relief(Gtk.ReliefStyle.NONE)
		button.set_can_focus(False)
		button.connect('clicked', self.on_click_open)
		box.pack_start(label, False, False, 60)
		box.pack_start(button, False, False, 15)
		self.add(box)
	
	def on_click_open(self, data):
		d = Gtk.FileChooserDialog("Open File", None, Gtk.FileChooserAction.OPEN, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL
				, Gtk.STOCK_OPEN, Gtk.ResponseType.ACCEPT))
		d.set_default_response(Gtk.ResponseType.ACCEPT)
		if d.run() == Gtk.ResponseType.ACCEPT:
			
			
			#self.handler = Handler(window)
			filename = d.get_filename()
			window = Window(filename)
			self.hide()
			#self.handler.notebook.new_tab(filename)
			#name = self.handler.split_filename(filename)
			#self.mw.set_title('Tahrir - ' + name)
			#doc = self.handler.notebook.get_current_doc()
			#doc.set_modified(False)
		d.destroy()
		
	def destroy_window(self, data):
		Gtk.main_quit()



from tahrir.toolbar import *
from tahrir.textview import *
from tahrir.notebook import *

class Handler:
	def __init__(self, mainwindow):
		self.mw = mainwindow
		self.filenames = {}
		self.set_notebook()
		self.set_toolbar()
		
	def set_window_title(self, filename):
		if filename != None:
			title = self.split_filename(filename)
			self.mw.set_title('Txtr - ' + title)
		else:
			self.mw.set_title('Txtr - Untitled Document' )
		
	def get_file_type(self):
		filename = self.notebook.get_current_filename()
		if filename != None:
			filetype = filename.split('.')
			filetype = filetype[-1]
		else:
			filetype = None
		return filetype
		
	def set_toolbar(self):
		self.toolbar = Toolbar(self)
	
	def set_toolbar_actions(self, boolean):
		self.toolbar.actions['save'].set_sensitive(boolean)
		self.toolbar.actions['cut'].set_sensitive(boolean)
		self.toolbar.actions['copy'].set_sensitive(boolean)
		self.toolbar.actions['paste'].set_sensitive(boolean)
		self.toolbar.search_entry.set_sensitive(boolean)
		self.toolbar.actions['search_forward'].set_sensitive(boolean)
		self.toolbar.actions['search_backward'].set_sensitive(boolean)
		
	def set_notebook(self):
		self.notebook = Notebook(self)
		
	def read_file(self, filename):
		f = open(filename, 'r')
		content = f.read()
		f.close()
		return content
	
	def confirm_exit(self):
		d = Gtk.Dialog()
		d.add_button('Close Without Saving', -1)
		d.add_button(Gtk.STOCK_CANCEL, 0)
		d.add_button(Gtk.STOCK_SAVE, 1)
		d.set_default_response(1)
		contentarea = d.get_content_area()
		msg = Gtk.Label('Save changes to document before closing ?\n')
		contentarea.pack_start(msg, True, True, 5)
		d.show_all()
		return d
		
	def on_click_new(self, data):
		self.notebook.new_tab(None)
		
	def split_filename(self, filename):
		name = filename.split('/')
		name = name[-1]
		return name
		
	def on_click_open(self, data):
		d = Gtk.FileChooserDialog("Open File", None, Gtk.FileChooserAction.OPEN, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL
				, Gtk.STOCK_OPEN, Gtk.ResponseType.ACCEPT))
		d.set_default_response(Gtk.ResponseType.ACCEPT)
		if d.run() == Gtk.ResponseType.ACCEPT:
			filename = d.get_filename()
			self.notebook.new_tab(filename)
			name = self.split_filename(filename)
			#self.mw.set_title('Tahrir - ' + name)
			doc = self.notebook.get_current_doc()
			doc.set_modified(False)
		d.destroy()
	
	def on_click_save(self, data):
		filename = self.notebook.get_current_filename()
		if filename != None:
			self.save_process(filename)
		else: 
			d = Gtk.FileChooserDialog("Save File", None, Gtk.FileChooserAction.SAVE, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL
				, Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
			d.set_do_overwrite_confirmation(True)
			if d.run() == Gtk.ResponseType.ACCEPT:
				filename = d.get_filename()
				self.notebook.filenames.append(filename)
				name = self.split_filename(filename)
				self.save_process(filename)
				self.mw.set_title('Tahrir - ' + name)
				self.notebook.update_tab_title(name)
			d.destroy()
				
	def save_process(self, filename):
		f = open(filename, 'w')
		doc = self.notebook.get_current_doc()
		contents = doc.get_text()
		f.write(contents)
		f.close()
		doc.set_modified(False)
		
	def on_click_undo(self, data):
		pass
		
	def on_click_redo(self, data):
		pass
		
	def on_click_cut(self, data):
		doc = self.notebook.get_current_doc()
		doc.cut_text()
		
	def on_click_copy(self, data):
		doc = self.notebook.get_current_doc()
		doc.copy_text()
		
	def on_click_paste(self, data):
		doc = self.notebook.get_current_doc()
		doc.paste_text()
	
	def on_search(self, data):
		text = self.toolbar.search_entry.get_text()
		doc = self.notebook.get_current_doc()
		doc.search_for_text(text)
		
	def on_click_search_forward(self, data):
		text = self.toolbar.search_entry.get_text()
		doc = self.notebook.get_current_doc()
		doc.search_forward(text)
		
	def on_click_search_backward(self, data):
		text = self.toolbar.search_entry.get_text()
		doc = self.notebook.get_current_doc()
		doc.search_backward(text)
		
	def on_click_about(self, data):
		pass
		
	def on_click_settings(self, data):
		pass
		
	def set_save_icon(self, boolean):
		self.toolbar.actions['save'].set_sensitive(boolean)



from distutils.core import *

setup(name='Txtr',
      version='1',
      author = 'Eslam Mostafa',
      author_email = 'cseslam@gmail.com',
      license = 'GPL3',
      description = 'A simple and neat text editor designed for gnu/linux',
      packages=['txtr',],
      scripts=['txtr/txtr',],
      )



from gi.repository import Gtk

class Doc(Gtk.TextView):
	def __init__(self, parent, handler, contents):
		super(Doc, self).__init__()
		self.parent = parent
		self.handler = handler
		self.contents = contents
		self.set_left_margin(5)
		self.buffer = self.get_buffer()
		self.set_text(contents)
		self.buffer.connect("changed", self.buffer_changed)
		self.buffer.connect("modified-changed", self.buffer_modified)
		self.set_highlighter()
		self.set_clipboard()
		self.set_search_tag()
		self.set_can_focus(True)
		self.grab_focus()
		self.show_all()
	
	def set_modified(self, boolean):
		self.buffer.set_modified(boolean)
	
	def buffer_modified(self, data):
		if self.buffer.get_modified() == True:
			self.handler.toolbar.actions['save'].set_sensitive(True)
		else:
			self.handler.toolbar.actions['save'].set_sensitive(False)
			
	def has_unsaved_changes(self):
		return self.buffer.get_modified()
		
	def get_text(self):
		contents = self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter(), True)
		return contents
		
	def set_text(self, text):
		self.buffer.set_text(text, -1)
		
	def set_clipboard(self):
		self.clipboard = Gtk.Clipboard()
		
	def set_search_tag(self):
		self.searchtag = self.buffer.create_tag('yellowBackground')
		self.searchtag.set_property('background', 'yellow')
		
	def cut_text(self):
		self.buffer.cut_clipboard(self.clipboard, False)
		
	def copy_text(self):
		self.buffer.copy_clipboard(self.clipboard)
		
	def paste_text(self):
		self.buffer.paste_clipboard(self.clipboard, None, True)
		
	def buffer_changed(self, data):
		self.handler.set_save_icon(True)
		self.parent.linenumbers.set_lines(self.buffer.get_line_count())
		self.do_highlight()
		
	def select_text(self, matchstart, matchend):
		self.buffer.select_range(matchstart, matchend)
	
	def find_text(self, text):		
		startiter = self.buffer.get_start_iter()
		result = []
		while True:
			found = startiter.forward_search(text, 0, None)
			if found:
				matchstart, matchend = found
				result.append((matchstart, matchend))
				startiter = matchend
			else:
				break
		return result
			
	def search_for_text(self, text):
		self.buffer.remove_tag(self.searchtag, self.buffer.get_start_iter(), self.buffer.get_end_iter())
		result = self.find_text(text)
		if len(result) != 0:
			self.matchstart = result[0][0]
			self.matchend = result[0][1]
			self.select_text(self.matchstart, self.matchend)
			for r in result:
				matchstart = r[0]
				matchend = r[1]
				self.buffer.apply_tag(self.searchtag, matchstart, matchend)
		else:
			pass
			
	def search_forward(self, text):
		if self.matchend:
			startiter = self.matchend
		if not self.matchend:
			startiter = self.buffer.get_start_iter()
		found = startiter.forward_search(text, 0, None)
		if found:
			self.matchstart, self.matchend = found
			self.select_text(self.matchstart, self.matchend)
			
	def search_backward(self, text):
		if self.matchstart:
			startiter = self.matchstart
		if not self.matchstart:
			startiter = self.buffer.get_end_iter()
		found = startiter.backward_search(text, 0, None)
		if found:
			self.matchstart, self.matchend = found
			self.select_text(self.matchstart, self.matchend)
			
	def set_highlighter(self):
		self.tags = {}
		self.tags['boldblue'] = self.buffer.create_tag('BoldBlue')
		self.tags['boldblue'].set_property('foreground', 'darkblue')
		self.tags['boldblue'].set_property('weight', 550)
		
		self.tags['blue'] = self.buffer.create_tag('blue')
		self.tags['blue'].set_property('foreground', 'blue')
		
		self.tags['darkgreen'] = self.buffer.create_tag('DarkGreen')
		self.tags['darkgreen'].set_property('foreground', 'darkgreen')
		
		self.tags['orange'] = self.buffer.create_tag('Orange')
		self.tags['orange'].set_property('foreground', 'orange')
		
		self.numbers = ['0','1','2','3','4','5','6','7','8','9']
		self.python = ['def', 'from', 'import', 'class', 'if', 'elif', 'for', 'else', 'return', 'break', 'while', 'do'] 
		self.c = ['function', 'this', '#include', 'class', 'if', 'for', 'else', 'return', 'break', 'exit', 'while', 'do'] 
	
	def do_highlight(self):
		if self.handler.get_file_type() == 'py':
			for code in self.python:
				result = self.find_text(code)
				if result:
					for iter in result:
						self.buffer.apply_tag(self.tags['boldblue'], iter[0], iter[1])
			for n in self.numbers:
				result = self.find_text(n)
				if result:
					for iter in result:
						self.buffer.apply_tag(self.tags['darkgreen'], iter[0], iter[1])
		elif self.handler.get_file_type() in ['c', 'cpp', 'h', 'cc']:
			for code in self.c:
				result = self.find_text(code)
				if result:
					for iter in result:
						self.buffer.apply_tag(self.tags['boldblue'], iter[0], iter[1])
			for n in self.numbers:
				result = self.find_text(n)
				if result:
					for iter in result:
						self.buffer.apply_tag(self.tags['darkgreen'], iter[0], iter[1])
		
